import six

from .metadata import parse_plugin_metadata

class TagServer():

  def __init__(self, multiplexer, plugin_name):
    self.plugin_name = plugin_name
    self._multiplexer = multiplexer

  def get_tags_response(self):
    """A route (HTTP handler) that returns a response with tags.
    Args:
      request: The werkzeug.Request object.
    Returns:
      A response that contains a JSON object. The keys of the object
      are all the runs. Each run is mapped to a (potentially empty)
      list of all tags that are relevant to this plugin.
    """

    all_runs = self._multiplexer.PluginRunToTagToContent(self.plugin_name)

    response = dict()
    for run, tag_to_content in six.iteritems(all_runs):
      response[run] = dict()

      for instance_tag, _ in six.iteritems(tag_to_content):
        
        # Make sure we only operate on user-defined tags here.
        tag = self._tag(run, instance_tag)
        meta, description = self._instance_tag_metadata(run, instance_tag)
        
        # Batch size must be defined, otherwise we don't know how many
        # samples were there.
        response[run][tag] = {"samples": meta.shape[0], "description": description}
    
    return response


  def collect_tensor_events(self, request, step=None):
    """Collects list of tensor events based on request."""
    run = request.args.get("run")
    tag = request.args.get("tag")

    tensor_events = []  # List of tuples (meta, tensor) that contain tag.
    for instance_tag in self._instance_tags(run, tag):
        tensors = self._multiplexer.Tensors(run, instance_tag)
        meta, description = self._instance_tag_metadata(run, instance_tag)
        tensor_events += [(meta, tensor, description) for tensor in tensors]

    if step is not None:
        tensor_events = [
            event for event in tensor_events if event[1].step == step
        ]
    else:
        # Make sure tensors sorted by step in ascending order.
        tensor_events = sorted(
            tensor_events, key=lambda tensor_data: tensor_data[1].step
        )

    return tensor_events




########### private methods #############
  def _instance_tag_metadata(self, run, instance_tag):
    """Gets the `GeoPluginData` proto for an instance tag."""
    summary_metadata = self._multiplexer.SummaryMetadata(run, instance_tag)
    content = summary_metadata.plugin_data.content
    metadata = parse_plugin_metadata(content)

    return metadata, summary_metadata.summary_description

  def _tag(self, run, instance_tag):
    """Gets the user-facing tag name for an instance tag."""
    meta, _ = self._instance_tag_metadata(run, instance_tag)
    return meta.name

  def _instance_tags(self, run, tag):
    """Gets the instance tag names for a user-facing tag."""
    index = self._multiplexer.GetAccumulator(run).PluginTagToContent(self.plugin_name)
    
    return [
      instance_tag
      for (instance_tag, content) in six.iteritems(index)
      if tag == parse_plugin_metadata(content).name
    ]



import torch
from tensorboard.compat.proto.summary_pb2 import Summary
from tensorboard.compat.proto.summary_pb2 import SummaryMetadata
from tensorboard.compat.proto.tensor_pb2 import TensorProto
from tensorboard.compat.proto.tensor_shape_pb2 import TensorShapeProto
import json

from . import metadata
from .plugin_data_pb2 import GeoPluginData

tag_history = {}

def add_geometry(
  writer,
  tag,
  vertices,
  vert_colors=None,
  faces=None,
  face_colors=None,
  features=None,
  feat_colors=None,
  config_dict=None,
  global_step=None,
  walltime=None,
  description=None):
  '''Add meshes or 3D point clouds to TensorBoard. The visualization is based on Three.js,
    so it allows users to interact with the rendered object. Besides the basic definitions
    such as vertices, faces, users can further provide camera parameter, lighting condition, etc.
    Please check https://threejs.org/docs/index.html#manual/en/introduction/Creating-a-scene for
    advanced usage.
    
    Args:
        tag (string): Data identifier
        vertices (torch.Tensor): List of the 3D coordinates of vertices.
        vert_colors (torch.Tensor): List of colors from 0 to 255 for each vertex.
        faces (torch.Tensor): Indices of vertices within each triangle. (Optional)
        face_color (torch.Tensor): List of colors for each sample in range [0,255]. (Optional)
        features (torch.Tensor): feature vectors for each vertex (Optional)
        feat_colors (torch.Tensor): List of colors from 0 to 255 for each feature. (Optional)
        config_dict: Dictionary with ThreeJS configuration. (Optional)
        global_step (int): Global step value to record (Optional)
        walltime (float): Optional override default walltime (time.time())
          seconds after epoch of event (Optional)
        description (string): A longform readable description of the summary data. Markdown is
          supported. (Optional)
    Shape:
        vertices: :math:`(B, N, 3)`. (batch, number_of_vertices, channels)
        vert_colors: :math:`(B, N, 3)`. (batch, number_of_vertices, 3) with type `uint8`
        faces: :math:`(B, N, 3)`. The values should lie in [0, number_of_vertices] for type `uint32`.
        face_colors: :math:`(B, 3)`. The values should lie in [0, 255] for type `uint8`.
        features: :math:`(B, N, 3)`. (batch, number_of_features, channels)
        feat_colors: :math:`(B, N, 3)`. (batch, number_of_features, 3) with type `uint8`
  '''
  torch._C._log_api_usage_once("tensorboard.logging.add_geometry")

  n_vert = vertices.shape[1]

  # check vertices
  if vertices.shape[2] != 3:
    raise ValueError("Vertices must be of shape [B, N, 3], but got %s" % str(vertices.shape))

  # check vert_colors
  if vert_colors is not None and vert_colors.shape[2] != 3:
    raise ValueError("Colors for vertices must be of shape [B, N, 3], but got %s" % str(vert_colors.shape))
  if vert_colors is not None and vert_colors.shape[1] != n_vert:
    raise ValueError("Number of vertices and colors for vertices must match, but got %s and %s" % (str(vertices.shape), str(vert_colors.shape)))

  # check faces
  if faces is not None and faces.shape[2] != 3:
    raise ValueError("Faces must be of shape [B, N, 3] but got %s" % str(faces.shape))

  # check face colors
  if face_colors is not None and vertices.shape[0] != face_colors.shape[0]:
    raise ValueError("Numbers of samples and colors for each sample must match, but got %s and %s" % (str(vertices.shape), str(face_colors)))

  # check features
  if features is not None and features.shape[2] != 3:
    raise ValueError("Features for vertices must be of shape [B, N, 3], but got %s" % str(features.shape))
  if features is not None and features.shape[1] != n_vert:
    raise ValueError("Number of vertices and features for vertices must match, but got %s and %s" % (str(vertices.shape), str(features.shape)))

  # check feat_colors
  if feat_colors is not None and feat_colors.shape[2] != 3:
    raise ValueError("Features for vertices must be of shape [B, N, 3], but got %s" % str(feat_colors.shape))
  if feat_colors is not None and feat_colors.shape[1] != n_vert:
    raise ValueError("Number of features and colors for features must match, but got %s and %s" % (str(features.shape), str(feat_colors.shape)))

  writer._get_file_writer()  \
        .add_summary(
          _geometry(
            tag,
            vertices,
            vert_colors,
            faces,
            face_colors,
            features,
            feat_colors,
            description,
            config_dict
          ),
          global_step=global_step
        )



def _geometry(
  tag,
  vertices,
  vert_colors=None,
  faces=None,
  face_colors=None,
  features=None,
  feat_colors=None,
  description=None,
  config_dict=None):
  '''Outputs a merged `Summary` protocol buffer with meshes/point clouds.
    Args:
      tag: A name for this summary operation.
      vertices: Tensor of shape `[B, number_of_vertices, 3]` representing the 3D
        coordinates of vertices.
      vert_colors: Tensor of shape `[B, number_of_vertices, 3]` representing colors for each
        vertex. Must be in range [0, 255] for type `uint8`.
      faces: Tensor of shape `[B, number_of_faces, 3]` containing indices of
        vertices within each triangle.
      feat_colors: Tensor of shape `[B, 3]` representing colors for each
        sample. Must be in range [0, 255] for type `uint8`.
      features: Tensor of shape `[B, number_of_vertices, 3]` containing 3D features for each
        vertex.
      feat_colors: Tensor of shape `[B, number_of_vertices, 3]` representing colors for each
        feature vector. Must be in range [0, 255] for type `uint8`.
      description: A longform readable description of the summary data. Markdown is
        supported.
      config_dict: Dictionary with ThreeJS classes names and configuration.
    Returns:
      Merged summary for mesh/point cloud representation.'''
  json_config = _get_json_config(config_dict)

  tensors = [
      (vertices, GeoPluginData.VERTICES),
      (vert_colors, GeoPluginData.VERT_COLORS),
      (faces, GeoPluginData.FACES),
      (face_colors, GeoPluginData.FACE_COLORS),
      (features, GeoPluginData.FEATURES),
      (feat_colors, GeoPluginData.FEAT_COLORS)
  ]
  tensors = [tensor for tensor in tensors if tensor[0] is not None]
  
  components = metadata.get_components_bitmask(
      [content_type for (tensor, content_type) in tensors]
    )

  summaries = []
  for tensor, content_type in tensors:
    summaries.append(
      _get_tensor_summary(
        tag,
        description,
        tensor,
        content_type,
        components,
        json_config
      )
    )

  return Summary(value=summaries)

def _get_tensor_summary(name, description, tensor, content_type, components, json_config):
  """Creates a tensor summary with summary metadata.
  Args:
    name: Uniquely identifiable name of the summary op. Could be replaced by
      combination of name and type to make it unique even outside of this
      summary.
    description: A longform readable description of the summary data. Markdown is
      supported.
    tensor: Tensor to display in summary.
    content_type: Type of content inside the Tensor.
    components: Bitmask representing present parts (vertices, colors, etc.) that
      belong to the summary.
    json_config: A string, JSON-serialized dictionary of ThreeJS classes
      configuration.
  Returns:
    Tensor summary with metadata.
  """

  tensor = torch.as_tensor(tensor)

  tensor_metadata = metadata.create_summary_metadata(
      name,
      description,
      content_type,
      components,
      tensor.shape,
      json_config)

  if (
    content_type == GeoPluginData.VERTICES or
    content_type == GeoPluginData.FEATURES  
  ):
    tensor = TensorProto(dtype='DT_FLOAT',
                          float_val=tensor.reshape(-1).float().tolist(),
                          tensor_shape=TensorShapeProto(dim=[
                              TensorShapeProto.Dim(size=tensor.shape[0]),
                              TensorShapeProto.Dim(size=tensor.shape[1]),
                              TensorShapeProto.Dim(size=tensor.shape[2]),
                          ]))
  elif (
    content_type == GeoPluginData.VERT_COLORS or
    content_type == GeoPluginData.FEAT_COLORS
  ):
    tensor = TensorProto(dtype='DT_UINT8',
                          int_val=tensor.reshape(-1).type(torch.uint8).tolist(),
                          tensor_shape=TensorShapeProto(dim=[
                              TensorShapeProto.Dim(size=tensor.shape[0]),
                              TensorShapeProto.Dim(size=tensor.shape[1]),
                              TensorShapeProto.Dim(size=tensor.shape[2]),
                          ]))
  elif content_type == GeoPluginData.FACE_COLORS:
    tensor = TensorProto(dtype='DT_UINT8',
                          int_val=tensor.reshape(-1).type(torch.uint8).tolist(),
                          tensor_shape=TensorShapeProto(dim=[
                              TensorShapeProto.Dim(size=tensor.shape[0]),
                              TensorShapeProto.Dim(size=tensor.shape[1])
                          ]))
  elif content_type == GeoPluginData.FACES:
    tensor = TensorProto(dtype='DT_INT32',
                          int_val=tensor.reshape(-1).int().tolist(),
                          tensor_shape=TensorShapeProto(dim=[
                              TensorShapeProto.Dim(size=tensor.shape[0]),
                              TensorShapeProto.Dim(size=tensor.shape[1]),
                              TensorShapeProto.Dim(size=tensor.shape[2]),
                          ]))

  tensor_summary = Summary.Value(
      tag=metadata.get_instance_name(name, content_type),
      tensor=tensor,
      metadata=tensor_metadata,
  )

  return tensor_summary


def _get_json_config(config_dict):
    """Parses and returns JSON string from python dictionary."""
    json_config = '{}'
    if config_dict is not None:
        json_config = json.dumps(config_dict, sort_keys=True)
    return json_config


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: plugin_data.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='plugin_data.proto',
  package='tensorboard_plugin_geo',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x11plugin_data.proto\x12\x16tensorboard_plugin_geo\"\xa7\x02\n\rGeoPluginData\x12\x0f\n\x07version\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12G\n\x0c\x63ontent_type\x18\x03 \x01(\x0e\x32\x31.tensorboard_plugin_geo.GeoPluginData.ContentType\x12\x13\n\x0bjson_config\x18\x05 \x01(\t\x12\r\n\x05shape\x18\x06 \x03(\x05\x12\x12\n\ncomponents\x18\x07 \x01(\r\"v\n\x0b\x43ontentType\x12\r\n\tUNDEFINED\x10\x00\x12\x0c\n\x08VERTICES\x10\x01\x12\t\n\x05\x46\x41\x43\x45S\x10\x02\x12\x0c\n\x08\x46\x45\x41TURES\x10\x03\x12\x0f\n\x0bVERT_COLORS\x10\x04\x12\x0f\n\x0b\x46\x41\x43\x45_COLORS\x10\x05\x12\x0f\n\x0b\x46\x45\x41T_COLORS\x10\x06\x62\x06proto3'
)



_GEOPLUGINDATA_CONTENTTYPE = _descriptor.EnumDescriptor(
  name='ContentType',
  full_name='tensorboard_plugin_geo.GeoPluginData.ContentType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNDEFINED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='VERTICES', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FACES', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FEATURES', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='VERT_COLORS', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FACE_COLORS', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FEAT_COLORS', index=6, number=6,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=223,
  serialized_end=341,
)
_sym_db.RegisterEnumDescriptor(_GEOPLUGINDATA_CONTENTTYPE)


_GEOPLUGINDATA = _descriptor.Descriptor(
  name='GeoPluginData',
  full_name='tensorboard_plugin_geo.GeoPluginData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='version', full_name='tensorboard_plugin_geo.GeoPluginData.version', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='tensorboard_plugin_geo.GeoPluginData.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='content_type', full_name='tensorboard_plugin_geo.GeoPluginData.content_type', index=2,
      number=3, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='json_config', full_name='tensorboard_plugin_geo.GeoPluginData.json_config', index=3,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='shape', full_name='tensorboard_plugin_geo.GeoPluginData.shape', index=4,
      number=6, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='components', full_name='tensorboard_plugin_geo.GeoPluginData.components', index=5,
      number=7, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _GEOPLUGINDATA_CONTENTTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=46,
  serialized_end=341,
)

_GEOPLUGINDATA.fields_by_name['content_type'].enum_type = _GEOPLUGINDATA_CONTENTTYPE
_GEOPLUGINDATA_CONTENTTYPE.containing_type = _GEOPLUGINDATA
DESCRIPTOR.message_types_by_name['GeoPluginData'] = _GEOPLUGINDATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GeoPluginData = _reflection.GeneratedProtocolMessageType('GeoPluginData', (_message.Message,), {
  'DESCRIPTOR' : _GEOPLUGINDATA,
  '__module__' : 'plugin_data_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard_plugin_geo.GeoPluginData)
  })
_sym_db.RegisterMessage(GeoPluginData)


# @@protoc_insertion_point(module_scope)




import os.path as osp
import json
import numpy as np
import six
from tensorboard.plugins import base_plugin
from tensorboard.util import tensor_util
import werkzeug
from werkzeug import wrappers

from .metadata import PLUGIN_NAME
from .tag_server import TagServer
from .data_server import DataServer

class GeoPlugin(base_plugin.TBPlugin):
  plugin_name = PLUGIN_NAME

  def __init__(self, context): # ...
    self._multiplexer = context.multiplexer
    self._tag_server = TagServer(self._multiplexer, self.plugin_name)
    self._data_server = DataServer(self._multiplexer, self._tag_server)
    self._logdir = context.logdir

  def get_plugin_apps(self):
    return {
      "/app": self._serve_js,
      "/app/*": self._serve_js,
      "/assets/*": self._serve_assets,
      "/tags": self._serve_tags,
      "/data": self._serve_data,
      "/geometries": self._serve_metadata,
      "/logdir": self._serve_logdir
    }

  ### Upon loading TensorBoard in browser
  def is_active(self):
    """Determines whether this plugin is active.
    This plugin is only active if TensorBoard sampled any summaries
    relevant to the geometry plugin.
    Returns:
      Whether this plugin is active.
    """
    all_runs = self._multiplexer.PluginRunToTagToContent(self.plugin_name)

    # The plugin is active if any of the runs has a tag relevant
    # to the plugin.
    return bool(self._multiplexer and any(six.itervalues(all_runs)))

  def frontend_metadata(self):
    return base_plugin.FrontendMetadata(es_module_path = "/app", tab_name = "Geometries")

  ### Route handling
  @wrappers.Request.application
  def _serve_js(self, request):
    base_path = osp.join(osp.dirname(__file__), "static", "bundle")
    req_path = request.path.split('/')
    req_path = req_path[len(req_path) - 1]

    if req_path == 'app':
      filepath = osp.join(base_path, "render.js")
    elif req_path == 'index':
      filepath = osp.join(base_path, 'index.js')
    
    with open(filepath) as infile:
      contents = infile.read()
    return werkzeug.Response(
      contents, content_type="application/javascript"
    )

  @wrappers.Request.application
  def _serve_assets(self, request):
    base_path = osp.join(osp.dirname(__file__), "static", "bundle", "assets")
    req_path = request.path.split('/')
    req_path = req_path[len(req_path) - 1]
    
    filepath = osp.join(base_path, req_path)
    file_size = osp.getsize(filepath)
    
    return werkzeug.Response(
      open(filepath, 'rb'),
      mimetype='application/octet-stream',
      headers=[
        ('Content-Length', str(file_size)),
        ('Content-Disposition', "attachment; filename=\"%s\"" % req_path),
      ],
      direct_passthrough=True)

  @wrappers.Request.application
  def _serve_tags(self, request):
    """A route (HTTP handler) that returns a response with tags.
    Args:
      request: The werkzeug.Request object.
    Returns:
      A response that contains a JSON object. The keys of the object
      are all the runs. Each run is mapped to a (potentially empty)
      list of all tags that are relevant to this plugin.
    """
    return werkzeug.Response(
      json.dumps(self._tag_server.get_tags_response()),
      content_type="application/json"
    )

  @wrappers.Request.application
  def _serve_data(self, request):
    """A route that returns data for particular summary of specified type.
    Data can represent vertices coordinates, vertices indices in faces,
    vertices colors and so on. Each mesh may have different combination of
    abovementioned data and each type/part of mesh summary must be served as
    separate roundtrip to the server.
    Args:
      request: werkzeug.Request containing content_type as a name of enum
        GeoPluginData.ContentType.
    Returns:
      werkzeug.Response either float32 or int32 data in binary format.
    """
    try:
      response = self._data_server.get_data_response(request, self.plugin_name)
    
    except ValueError as err:
      raise err
      res = werkzeug.Response("Bad content_type", "text/plain")
      res.status_code = 400
      return res

    res = werkzeug.Response(response, "arraybuffer")
    res.status_code = 200
    return res

  @wrappers.Request.application
  def _serve_logdir(self, request):
    return werkzeug.Response(
      json.dumps({ 'logdir': self._logdir}), content_type="application/javascript"
    )

  @wrappers.Request.application
  def _serve_metadata(self, request):
      """A route that returns the mesh metadata associated with a tag.
      Metadata consists of wall time, type of elements in tensor, scene
      configuration and so on.
      Args:
        request: The werkzeug.Request object.
      Returns:
        A JSON list of mesh data associated with the run and tag
        combination.
      """
      tensor_events = self._tag_server.collect_tensor_events(request)

      # We convert the tensor data to text.
      response = [{
          "wall_time": event.wall_time,
          "step": event.step,
          "content_type": meta.content_type,
          "components": meta.components,
          "config": meta.json_config,
          "data_shape": list(meta.shape),
          "description": description,
      } for meta, event, description in tensor_events]

      res = werkzeug.Response(json.dumps(response), "application/json")
      res.status_code = 200
      return res




import numpy as np
from tensorboard.util import tensor_util

from .plugin_data_pb2 import GeoPluginData

class DataServer():

  def __init__(self, multiplexer, tag_server):
    self._multiplexer = multiplexer
    self._tag_server = tag_server

  def get_data_response(self, request, plugin_name):
    """A route that returns data for particular summary of specified type.
    Data can represent vertices coordinates, vertices indices in faces,
    vertices colors and so on. Each mesh may have different combination of
    abovementioned data and each type/part of mesh summary must be served as
    separate roundtrip to the server.
    Args:
      request: werkzeug.Request containing content_type as a name of enum
        GeoPluginData.ContentType.
      plugin_name: identifier of the plugin
    Returns:
      data response
    """
    step = float(request.args.get("step", 0.0))
    content_type = request.args.get("content_type")
    content_type = GeoPluginData.ContentType.Value(content_type)
    
    tensor_events = self._collect_tensor_events(request, step)

    response = [
      self._get_tensor_data(tensor)
      for meta, tensor in tensor_events
      if meta.content_type == content_type
    ]

    np_type = {
      GeoPluginData.VERTICES: np.float32,
      GeoPluginData.FACES: np.int32,
      GeoPluginData.FEATURES: np.float32,
      GeoPluginData.VERT_COLORS: np.uint8,
      GeoPluginData.FACE_COLORS: np.uint8,
      GeoPluginData.FEAT_COLORS: np.uint8,
    }

    response = np.array(response, dtype=np_type[content_type])
    print('got array for %s - %s - %s - %d of shape %s' % (request.args.get("run"), request.args.get("tag"), request.args.get("step"), content_type, response.shape))
    # Looks like reshape can take around 160ms, so why not store it reshaped.
    return response.reshape(-1).tobytes()





######### private methods ########
  def _collect_tensor_events(self, request, step):
    """Collects list of tensor events based on request."""
    run = request.args.get("run")
    tag = request.args.get("tag")

    tensor_events = []  # List of tuples (meta, tensor) that contain tag.
    for instance_tag in self._tag_server._instance_tags(run, tag):
      tensors = self._multiplexer.Tensors(run, instance_tag)
      meta, _ = self._tag_server._instance_tag_metadata(run, instance_tag)
      tensor_events += [(meta, tensor) for tensor in tensors]

    if step is not None:
      tensor_events = [
        event for event in tensor_events if event[1].step == step
      ]
    else:
      # Make sure tensors sorted by step in ascending order.
      tensor_events = sorted(
        tensor_events, key=lambda tensor_data: tensor_data[1].step
      )

    return tensor_events

  def _get_tensor_data(self, event):
    """Convert a TensorEvent into a JSON-compatible response."""
    data = tensor_util.make_ndarray(event.tensor_proto)
    return data.tolist()  



import os
from time import sleep

from torch.utils.tensorboard import SummaryWriter

os.sys.path.append('..')
from tensorboard_plugin_geometry.summary import add_geometry

from tests.utils import get_rand_vecs

SummaryWriter.add_geometry = add_geometry

def test(log_dir):
  writer = SummaryWriter(log_dir=log_dir)
  
  for i in range(30):
    print('added %d to %s' % (i, log_dir))
    pos, _ = get_rand_vecs(100)
    writer.add_geometry(
      'test_reload',
      pos.reshape(1, 100, 3),
      global_step=i)
    sleep(10)

  writer.close()

i = 0

for i in range(100):
  test('./logs/test_%d' % i)


import os

from torch.utils.tensorboard import SummaryWriter

from tests.point_clouds import tests as PointCloudTests
from tests.meshes import tests as MeshTests
from tests.parameters import tests as ParameterTests

os.sys.path.append('..')
from tensorboard_plugin_geometry.summary import add_geometry

SummaryWriter.add_geometry = add_geometry

def test(log_name, callback):
  log_dir = log_name
  writer = SummaryWriter(log_dir=log_dir)
  
  suite = callback(writer)

  writer.close()
  return suite

def accumulate_results(suites):
  succeeded = 0
  failed = 0
  for suite in suites:
    succeeded += suite.succeeded
    failed += suite.failed
  
  return succeeded, failed

print(' ')
print('start testing')
s1 = test('./logs/parameters', ParameterTests)
s2 = test('./logs/point_clouds', PointCloudTests)
s3 = test('./logs/meshes', MeshTests)

succeeded, failed = accumulate_results([s1, s2, s3])
print('')
print(s1.divider % '')
print('succeeded: %d     failed: %d' % (succeeded, failed))
print('')



import setuptools

__version__ = '0.4.0'

url = 'https://github.com/jannessm/tensorboard-plugin-geometry'

with open("README.md", "r") as fh:
	long_description = fh.read()

def parse_requirements(filename):
	"""Load requirements from a pip requirements file."""
	lineiter = (line.strip() for line in open(filename))
	return [line for line in lineiter if line and not line.startswith("#")]

setuptools.setup(
    name="tensorboard_plugin_geometry",
    version=__version__,
    author='Jannes Magnusson',
    author_email="j-magnusson@t-online.de",
    url=url,
    description="Tensorboard plugin to visualize 3D data with 3D features.",
	long_description=long_description,
	long_description_content_type="text/markdown",
    license="GNU GPL",
	install_requires=parse_requirements("requirements.txt"),
    packages=["tensorboard_plugin_geometry"],
    package_data={
        "tensorboard_plugin_geometry": ["static/bundle/*"],
        "tensorboard_plugin_geometry": ["static/bundle/assets/*"],
    },
    entry_points={
        "tensorboard_plugins": [
            "geometry = tensorboard_plugin_geometry.plugin:GeoPlugin",
        ],
    },
    keywords='tensorboard mesh geometries plugin'
)


from tensorboard.compat.proto.summary_pb2 import SummaryMetadata

from .plugin_data_pb2 import GeoPluginData

PLUGIN_NAME = "geometries"

def get_current_version():
  return 0

def get_components_bitmask(content_types):
  """Creates bitmask for all existing components of the summary.
  Args:
    content_type: list of GeoPluginData.ContentType,
      representing all components related to the summary.
  Returns: bitmask based on passed tensors.
  """
  components = 0
  for content_type in content_types:
    if content_type == GeoPluginData.UNDEFINED:
      raise ValueError("Cannot include UNDEFINED content type in mask.")
    components = components | (1 << content_type)
  return components

def create_summary_metadata(name, description, content_type, components, shape, json_config):
  """Creates summary metadata which defined at GeoPluginData proto.
    Arguments:
      name: Original merged (summaries of different types) summary name.
      description: Description displayed by TensorboardUI
      content_type: Value from GeoPluginData enum describing data.
      components: mask representing present parts (vertices, features, etc.) that
        belong to the summary.
      shape: list of dimensions sizes of the tensor.
      json_config: A string, JSON-serialized dictionary of ThreeJS classes
        configuration.
    Returns:
      A `summary_pb2.SummaryMetadata` protobuf object.
  """
  
  # Shape should be at least BxNx3 where B represents the batch dimensions
  # and N - the number of points, each with x,y,z coordinates.
  if len(shape) != 3 and content_type != GeoPluginData.FACE_COLORS:
    raise ValueError("Tensor shape should be of shape BxNx3, but got %s." % str(shape))

  if len(shape) != 2 and content_type == GeoPluginData.FACE_COLORS:
    raise ValueError("Face color tensor must be of shape Bx3, but got %s." % str(shape))

  geo_plugin_data = GeoPluginData(
    version=get_current_version(),
    name=name,
    content_type=content_type,
    components=components,
    shape=shape,
    json_config=json_config,
  )
  content = geo_plugin_data.SerializeToString()
  plugin_data = SummaryMetadata.PluginData(
    plugin_name=PLUGIN_NAME, content=content
  )


  return SummaryMetadata(
    display_name=name,  # Will not be used in TensorBoard UI.
    summary_description=description,
    plugin_data=plugin_data,
  )

def parse_plugin_metadata(content):
  """Parse summary metadata to a Python object.
  Arguments:
    content: The `content` field of a `SummaryMetadata` proto
      corresponding to the mesh plugin.
  Returns:
    A `GeoPluginData` protobuf object.
  Raises: Error if the version of the plugin is not supported.
  """
  if not isinstance(content, bytes):
    raise TypeError("Content type must be bytes.")
  result = GeoPluginData.FromString(content)
  
  if not 0 <= result.version <= get_current_version():
    raise ValueError(
        "Unknown metadata version: %s. The latest version known to "
        "this build of TensorBoard is %s; perhaps a newer build is "
        "available?" % (result.version, get_current_version())
    )
  
  return result

def get_instance_name(tag, content_type):
  return "%s_%s" % (
    tag,
    GeoPluginData.ContentType.Name(content_type)
  )
